@@ -8,7 +8,9 @@ from collections import defaultdict
 from . import __version__
 from .grpc_handler import GrpcHandler
 from .http_handler import HttpHandler
-from milvus.client.exceptions import ConnectionPoolError
+from milvus.client.exceptions import ConnectionPoolError, NotConnectError, VersionError
+
+support_versions = ('0.9.0', '0.9.1', '0.10.0')
 
 
 class Duration:
@@ -32,7 +34,7 @@ class Duration:
 
 
 class ConnectionRecord:
-    def __init__(self, uri, recycle, handler="GRPC", conn_id=-1, **kwargs):
+    def __init__(self, uri, handler="GRPC", conn_id=-1, pre_ping=False, **kwargs):
         '''
         @param uri server uri
         @param recycle int, time period to recycle connection.
@@ -40,14 +42,15 @@ class ConnectionRecord:
         '''
         self._conn_id = conn_id
         self._uri = uri
-        self.recycle = recycle
+        # self.recycle = recycle
+        self._pre_ping = pre_ping
         self._last_use_time = time.time()
         self._kw = kwargs
 
         if handler == "GRPC":
-            self._connection = GrpcHandler(uri=uri)
+            self._connection = GrpcHandler(uri=uri, pre_ping=self._pre_ping)
         elif handler == "HTTP":
-            self._connection = HttpHandler(uri=uri)
+            self._connection = HttpHandler(uri=uri, pre_ping=self._pre_ping)
         else:
             raise ValueError("Unknown handler type. Use GRPC or HTTP")
 
@@ -55,36 +58,47 @@ class ConnectionRecord:
         ''' Return a available connection. If connection is out-of-date,
         return new one.
         '''
-        if self._kw.get("pre_ping", False):
+        self._last_use_time = time.time()
+        if self._pre_ping:
             self._connection.ping(timeout=2)
         return self._connection
 
 
 class ConnectionPool:
-    def __init__(self, uri, pool_size=10, recycle=-1, wait_timeout=10, **kwargs):
+    def __init__(self, uri, pool_size=10, wait_timeout=10, try_connect=True, **kwargs):
         # Asynchronous queue to store connection
-        self._pool = queue.Queue()
+        self._pool = queue.Queue(maxsize=pool_size)
         self._uri = uri
         self._pool_size = pool_size
-        self._recycle = recycle
         self._wait_timeout = wait_timeout
+        if try_connect:
+            self._max_retry = kwargs.get("max_retry", 3)
 
         # Record used connection number.
         self._used_conn = 0
         self._condition = threading.Condition()
         self._kw = kwargs
 
-        #
         self.durations = defaultdict(list)
+
+        self._try_connect = try_connect
         self._prepare()
 
     def _prepare(self):
         conn = self.fetch()
         with self._condition:
-            conn.client().ping()
+            if self._try_connect:
+                conn.client().ping(max_retry=self._max_retry)
+
             status, version = conn.client().server_version(timeout=1)
-            if version not in ('0.8.0', '0.9.0'):
-                raise ValueError("Version of python SDK({}) not match that of server{}.".format(__version__, version))
+            if not status.OK():
+                raise NotConnectError("Cannot check server version: {}".format(status.message))
+            if version not in support_versions:
+                raise VersionError(
+                    "Version of python SDK({}) not match that of server{}, excepted is {}".format(__version__,
+                                                                                                  version,
+                                                                                                  support_versions))
+        conn.close()
 
     def _inc_used(self):
         with self._condition:
@@ -111,7 +125,7 @@ class ConnectionPool:
 
     def _create_connection(self):
         with self._condition:
-            conn = ConnectionRecord(self._uri, self._recycle, conn_id=self._used_conn - 1, **self._kw)
+            conn = ConnectionRecord(self._uri, conn_id=self._used_conn - 1, **self._kw)
             return ScopedConnection(self, conn)
 
     def _inc_connection(self):
@@ -221,10 +235,13 @@ class ScopedConnection:
         return self._connection._conn_id
 
     def close(self):
+        self._closed = True
+        if not self._pool:
+            return
+
         self._connection and self._pool.release(self._connection)
         self._connection = None
         if self._duration:
             self._duration.stop()
             self._pool.record_duration(self._connection, self._duration)
         self._duration = None
-        self._closed = True
