@@ -34,9 +34,6 @@ def deprecated(func):
 def check_connect(func):
     @functools.wraps(func)
     def inner(self, *args, **kwargs):
-        # if not self.connected():
-        #     raise NotConnectError('Please connect to the server first')
-
         return func(self, *args, **kwargs)
 
     return inner
@@ -45,7 +42,7 @@ def check_connect(func):
 def _pool_args(**kwargs):
     pool_kwargs = dict()
     for k, v in kwargs.items():
-        if k in ("pool_size", "wait_timeout", "recycle"):
+        if k in ("pool_size", "wait_timeout", "handler", "try_connect", "pre_ping"):
             pool_kwargs[k] = v
 
     return pool_kwargs
@@ -72,7 +69,7 @@ def _set_uri(host, port, uri, handler="GRPC"):
                          "\t(uri = ${URI})\n")
 
     if not is_legal_host(_host) or not is_legal_port(_port):
-        raise ParamError("host or port is illegal")
+        raise ParamError("host {} or port {} is illegal".format(_host, _port))
 
     return "{}{}:{}".format(uri_prefix, str(_host), str(_port))
 
@@ -84,73 +81,40 @@ class Milvus:
         self._status = None
         self._connected = False
         self._handler = handler
-        # self._stub = None
 
         _uri = kwargs.get('uri', None)
         pool_uri = _set_uri(host, port, _uri, self._handler)
-        pool_kwargs = _pool_args(**kwargs)
+        pool_kwargs = _pool_args(handler=handler, **kwargs)
         self._pool = ConnectionPool(pool_uri, **pool_kwargs)
         # store extra key-words arguments
         self._kw = kwargs
         self._hooks = collections.defaultdict()
 
-        # if host or port or _uri:
-        #     kw = copy.deepcopy(kwargs)
-        #     kw.pop('uri', None)
-        #     self._init(host, port, _uri, **kw)
-        #     self._status = Status()
-
     def __enter__(self):
-        return self._pool.fetch()
+        self._conn = self._pool.fetch()
+        return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        pass
-        # self.__conn.close()
-        # self._stub = None
-        # self._status = None
-
-    def __getattr__(self, item):
-        if item == "add_vectors":
-            return self.insert
-        if item == "search_vectors":
-            return self.search
-        if item == "search_vectors_in_files":
-            return self.search_in_files
-
-        if item in self._kw.keys():
-            return self._kw.get(item, None)
-        raise AttributeError("Attribute {} not exists".format(item))
+        self._conn.close()
+        self._conn = None
 
     def __del__(self):
         return self.close()
 
-    # def _init(self, host, port, uri, **kwargs):
-    #     handler = kwargs.get("handler", "GRPC")
-    #     self._uri = self._set_uri(host, port, uri, handler)
-    #     self._stub = GrpcHandler(None, None, uri=self._uri)
-    #     if len(self._hooks) > 0:
-    #         self._stub.set_hook(**self._hooks)
-
     def _connection(self):
         return self._pool.fetch()
 
-    # def _get_connection
-
+    @deprecated
     def set_hook(self, **kwargs):
+        """
+        Deprecated
+        """
         # TODO: may remove it.
         if self._stub:
             return self._stub.set_hook(**kwargs)
 
         self._hooks.update(kwargs)
 
-    # @property
-    # def status(self):
-    #     return self._handler.status
-    # pass
-    #
-    # def connected(self):
-    #     return True
-
     @property
     def name(self):
         return self._name
@@ -159,30 +123,15 @@ class Milvus:
     def handler(self):
         return self._handler
 
-    # def ping(self, timeout=2):
-    #     """
-    #     Check if server is accessible
-    #
-    #     """
-    #     if self.handler == "GRPC":
-    #         from .grpc_handler import set_uri, connect
-    #         _addr = set_uri(None, None, self._uri)
-    #         return connect(_addr, timeout)
-    #     if self.handler == "HTTP":
-    #         self._stub.connect()
-    #
-    #     return True
-
     @deprecated
     def connect(self, host=None, port=None, uri=None, timeout=2):
+        """
+        Deprecated
+        """
         if self.connected() and self._connected:
             return Status(message="You have already connected {} !".format(self._uri),
                           code=Status.CONNECT_FAILED)
 
-        # if (host or port or uri) or not self._uri:
-        #     kw = copy.deepcopy(self._kw)
-        #     kw.pop('uri', None)
-        #     self._init(host, port, uri, **kw)
         if self._stub is None:
             self._init(host, port, uri, handler=self._handler)
 
@@ -193,14 +142,22 @@ class Milvus:
 
     @deprecated
     def connected(self):
+        """
+        Deprecated
+        """
         return True if self._status and self._status.OK() else False
 
     @deprecated
     def disconnect(self):
+        """
+        Deprecated
+        """
         pass
-    #
+
     def close(self):
-        # self._pool.terminate()
+        """
+        Close client instance
+        """
         self._pool = None
 
     def client_version(self):
@@ -228,15 +185,15 @@ class Milvus:
 
     def server_version(self, timeout=10):
         """
-       Returns the version of the Milvus server.
+        Returns the version of the Milvus server.
 
-       :return:
+        :return:
            Status: Whether the operation is successful.
 
            str : Version of the Milvus server.
 
-       :rtype: (Status, str)
-       """
+        :rtype: (Status, str)
+        """
 
         return self._cmd("version", timeout)
 
@@ -316,7 +273,7 @@ class Milvus:
             return handler.has_collection(collection_name, timeout)
 
     @check_connect
-    def describe_collection(self, collection_name, timeout=10):
+    def get_collection_info(self, collection_name, timeout=10):
         """
         Returns information of a collection.
 
@@ -326,6 +283,7 @@ class Milvus:
         :returns: (Status, table_schema)
             Status: indicate if query is successful
             table_schema: return when operation is successful
+
         :rtype: (Status, TableSchema)
         """
         check_pass_param(collection_name=collection_name)
@@ -333,7 +291,7 @@ class Milvus:
             return handler.describe_collection(collection_name, timeout)
 
     @check_connect
-    def count_collection(self, collection_name, timeout=10):
+    def count_entities(self, collection_name, timeout=10):
         """
         Returns the number of vectors in a collection.
 
@@ -350,14 +308,14 @@ class Milvus:
             return handler.count_collection(collection_name, timeout)
 
     @check_connect
-    def show_collections(self, timeout=10):
+    def list_collections(self, timeout=10):
         """
-        Returns information of all collections.
+        Returns collection list.
 
         :return:
             Status: indicate if this operation is successful
 
-            collections: list of table names, return when operation
+            collections: list of collection names, return when operation
                     is successful
         :rtype:
             (Status, list[str])
@@ -366,19 +324,28 @@ class Milvus:
             return handler.show_collections(timeout)
 
     @check_connect
-    def collection_info(self, collection_name, timeout=10):
-        # TODO: need check collection_name here
+    def get_collection_stats(self, collection_name, timeout=10):
+        """
+        Returns collection statistics information
+
+        :return:
+            Status: indicate if this operation is successful
+
+            statistics: statistics information
+        :rtype:
+            (Status, dict)
+        """
         check_pass_param(collection_name=collection_name)
         with self._connection() as handler:
             return handler.show_collection_info(collection_name, timeout)
 
     @check_connect
-    def preload_collection(self, collection_name, timeout=None):
+    def load_collection(self, collection_name, timeout=None):
         """
         Loads a collection for caching.
 
         :type collection_name: str
-        :param collection_name: table to preload
+        :param collection_name: collection to load
 
         :returns:
             Status:  indicate if invoke is successful
@@ -393,7 +360,7 @@ class Milvus:
         Deletes a collection by name.
 
         :type  collection_name: str
-        :param collection_name: Name of the table being deleted
+        :param collection_name: Name of the collection being deleted
 
         :return: Status, indicate if operation is successful
         :rtype: Status
@@ -432,7 +399,8 @@ class Milvus:
         :rtype: (Status, list(int))
         """
         if kwargs.get("insert_param", None) is not None:
-            return self._stub.insert(None, None, timeout=timeout, **kwargs)
+            with self._connection() as handler:
+                return handler.insert(None, None, timeout=timeout, **kwargs)
 
         check_pass_param(collection_name=collection_name, records=records)
         partition_tag is not None and check_pass_param(partition_tag=partition_tag)
@@ -448,14 +416,27 @@ class Milvus:
             return handler.insert(collection_name, records, ids, partition_tag, params, timeout, **kwargs)
 
     @check_connect
-    def get_vectors_by_ids(self, collection_name, ids, timeout=None):
+    def get_entity_by_id(self, collection_name, ids, timeout=None):
+        """
+        Returns raw vectors according to ids.
+
+        :param collection_name: Name of the collection
+        :type collection_name: str
+
+        :param ids: list of vector id
+        :type ids: list
+
+        :returns:
+            Status: indicate if invoke is successful
+
+        """
         check_pass_param(collection_name=collection_name, ids=ids)
 
         with self._connection() as handler:
             return handler.get_vectors_by_ids(collection_name, ids, timeout=timeout)
 
     @check_connect
-    def get_vector_ids(self, collection_name, segment_name, timeout=None):
+    def list_id_in_segment(self, collection_name, segment_name, timeout=None):
         check_pass_param(collection_name=collection_name)
         check_pass_param(collection_name=segment_name)
         with self._connection() as handler:
@@ -495,7 +476,7 @@ class Milvus:
             return handler.create_index(collection_name, _index_type, params, timeout, **kwargs)
 
     @check_connect
-    def describe_index(self, collection_name, timeout=10):
+    def get_index_info(self, collection_name, timeout=10):
         """
         Show index information of a collection.
 
@@ -557,12 +538,26 @@ class Milvus:
 
     @check_connect
     def has_partition(self, collection_name, partition_tag):
+        """
+        Check if specified partition exists.
+
+        :param collection_name: target table name.
+        :type  collection_name: str
+
+        :param partition_tag: partition tag.
+        :type  partition_tag: str
+
+        :return:
+            Status: Whether the operation is successful.
+            exists: If specified partition exists
+
+        """
         check_pass_param(collection_name=collection_name, partition_tag=partition_tag)
         with self._connection() as handler:
             return handler.has_partition(collection_name, partition_tag)
 
     @check_connect
-    def show_partitions(self, collection_name, timeout=10):
+    def list_partitions(self, collection_name, timeout=10):
         """
         Show all partitions in a collection.
 
@@ -642,21 +637,9 @@ class Milvus:
             return handler.search(collection_name, top_k, query_records, partition_tags, params, timeout, **kwargs)
 
     @check_connect
-    def search_by_ids(self, collection_name, ids, top_k, partition_tags=None, params=None, timeout=None, **kwargs):
-        check_pass_param(collection_name=collection_name, topk=top_k, ids=ids)
-        partition_tags is not None and check_pass_param(partition_tag_array=partition_tags)
-
-        params = dict() if params is None else params
-        if not isinstance(params, dict):
-            raise ParamError("Params must be a dictionary type")
-
-        with self._connection() as handler:
-            return handler.search_by_ids(collection_name, ids, top_k, partition_tags, params, timeout, **kwargs)
-
-    @check_connect
-    def search_in_files(self, collection_name, file_ids, query_records, top_k, params=None, timeout=None, **kwargs):
+    def search_in_segment(self, collection_name, file_ids, query_records, top_k, params=None, timeout=None, **kwargs):
         """
-        Searches for vectors in specific files of a collection.
+        Searches for vectors in specific segments of a collection.
 
         The Milvus server stores vector data into multiple files. Searching for vectors in specific files is a
         method used in Mishards. Obtain more detail about Mishards, see
@@ -694,10 +677,18 @@ class Milvus:
                                            query_records, top_k, params, timeout, **kwargs)
 
     @check_connect
-    def delete_by_id(self, collection_name, id_array, timeout=None):
+    def delete_entity_by_id(self, collection_name, id_array, timeout=None):
         """
         Deletes vectors in a collection by vector ID.
 
+        :param collection_name: Name of the collection.
+        :type  collection_name: str
+
+        :param id_array: list of vector id
+        :type  id_array: list[int]
+
+        :return:
+            Status: Whether the operation is successful.
         """
         check_pass_param(collection_name=collection_name, ids=id_array)
         with self._connection() as handler:
@@ -759,11 +750,3 @@ class Milvus:
 
         return self._cmd(cmd)
 
-    # In old version of pymilvus, some methods are different from the new.
-    # apply alternative method name for compatibility
-
-    # get_collection_row_count = count_collection
-    # delete_collection = drop_collection
-    # add_vectors = insert
-    # search_vectors = search
-    # search_vectors_in_files = search_in_files
